unit PS3000FM;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls;

type
  TfmPS3000 = class(TForm)
  lstResults: TListBox;
  btnStart  : TButton;
  tmrSamples: TTimer;
  bntStart  : TButton;
  btnStop   : TButton;
  procedure btnStopClick(Sender: TObject);
  procedure btnStartClick(Sender: TObject);
  procedure tmrSamplesTimer(Sender: TObject);
  procedure bntStartClick(Sender: TObject);
  procedure FormCreate(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  fmPS3000: TfmPS3000;

implementation

const
  max_values  = 100;

{$I ps3000.inc}

var
  ps3000_handle : smallint;
  run_fast      : boolean;
  run_slow      : boolean;
  mv_a          : longint;
  mv_b          : longint;
  ns            : longint;
  slow_collect  : boolean;
  no_of_slow    : integer;
  streaming     : boolean;

  {$R *.DFM}

procedure TfmPS3000.btnStartClick(Sender: TObject);
var
  timebase           : smallint;
  no_of_samples      : longint;
  time_interval      : longint;
  time_units         : smallint;
  time_indisposed_ms : longint;
  oversample         : smallint;
  max_samples	     : longint;
  overflow           : smallint;
  s1                 : string[6];
  i                  : Integer;
  times              : array [1..max_values] of longint;
  values_a           : array [1..max_values] of smallint;
  values_b           : array [1..max_values] of smallint;
  values_c           : array [1..max_values] of smallint;
  values_d           : array [1..max_values] of smallint;

begin
  if ps3000_handle > 0 then
    begin
      lstResults.Items.Clear;
      lstResults.Items.Add ( 'Started' );
      timebase := 13;
      no_of_samples := max_values;

      ps3000_set_ets ( ps3000_handle, 0, 0, 0 );
      ps3000_set_channel ( ps3000_handle, 0, false, true, 10 );
      ps3000_set_channel ( ps3000_handle, 1, true, false, 10 );

      oversample := 1;
      ps3000_get_timebase ( ps3000_handle, timebase, no_of_samples, time_interval, time_units, oversample, max_samples );

      ps3000_run_block ( ps3000_handle, no_of_samples, timebase, oversample, time_indisposed_ms );
      while ( ps3000_ready (ps3000_handle)=0 ) do
        begin
        end;

      ps3000_stop ( ps3000_handle );

      ps3000_get_times_and_values ( ps3000_handle, times, values_a, values_b, values_c, values_d, overflow, time_units, no_of_samples );

      for i := 1 to no_of_samples do
        begin
          str ( values_b[i] * 20000 /max_ps3000_value:6:2,s1 );
          lstResults.items.Add ( s1 );
        end;
    end;

end;

procedure TfmPS3000.tmrSamplesTimer(Sender: TObject);
var
  buffer             : array [1..max_values] of smallint;
  ok                 : boolean;
  s1                 : string[6];
  i                  : longint;
  no_of_values       : longint;
  time_indisposed_ms : longint;
  oversample         : smallint;
  overflow           : smallint;
  mv_a               : smallint;
  mv_b               : smallint;
  oki                : smallint;
  times              : array [1..max_values] of longint;
  values_a           : array [1..max_values] of smallint;
  values_b           : array [1..max_values] of smallint;
  values_c           : array [1..max_values] of smallint;
  values_d           : array [1..max_values] of smallint;

begin
  if slow_collect then
     begin
       no_of_values := max_values;
       if not streaming then
         begin
           ps3000_set_ets ( ps3000_handle, 0, 0, 0 );
           { set channel A off }
           ps3000_set_channel ( ps3000_handle, 0, false, false, 10 );
           { set channel B on }
           ps3000_set_channel ( ps3000_handle, 1, true, false, 10 );
           ps3000_set_trigger ( ps3000_handle, 5, 0, false, 0, 0 );
           ps3000_run_streaming ( ps3000_handle, 10, no_of_values, 0 );
           streaming := True;
         end;
       no_of_values := ps3000_get_values( ps3000_handle, values_a, values_b, values_c, values_d, overflow, no_of_values );
       mv_a := 20000;
       mv_b := 20000;
       for i := 1 to no_of_values do
         begin
           str ( values_b[i] * 20000 /max_ps3000_value:6:2,s1 );
           lstResults.items.Add ( s1 );
         end;
     end;
end;

procedure TfmPS3000.bntStartClick(Sender: TObject);

begin
  if ps3000_handle > 0 then
    begin
      btnStart.Enabled := False;
      slow_collect := True;
      lstResults.Items.Clear;
      streaming := False;
    end;

end;

procedure TfmPS3000.btnStopClick(Sender: TObject);

begin
  ps3000_stop ( ps3000_handle );
  bntStart.Enabled := True;
  slow_collect := True;
end;

procedure TfmPS3000.FormCreate(Sender: TObject);

var
  i           : integer;
  lth         : integer;
  str         : string;
  str2        : array [1..80] of char;

begin
  lstResults.Items.Clear;
  lstResults.Items.Add ( 'Opening ps3000...' );
  ps3000_handle := ps3000_open_unit ( );
  if ps3000_handle > 0 then
    begin
      lstResults.Items.Add ('ps3000 opened ok');
      for i := 0 to 4 do
        begin
          lth := ps3000_get_unit_info ( ps3000_handle, str2, sizeof (str2), i );
	  SetString ( str, pchar (@str2), lth );
	  lstResults.Items.Add ( str );
        end
    end
  else
    begin
      lstResults.Items.Add ('Unable to open ps3000');
    end

end;

end.
